# coding: utf-8

"""
    Amber API Server

    Boon Logic Amber API server

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class MAutotune(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'version_number': 'VersionNumber',
        'm_autotuning_in_progress': 'bool',
        'm_percent_complete': 'float',
        'm_autotuning_succeeded': 'bool',
        'm_num_patterns_to_autotune': 'float',
        'm_error_string_buffer': 'str',
        'm_features_to_tune_array': 'list[bool]',
        'm_ncp': 'MNCP',
        'm_ap': 'MAP'
    }

    attribute_map = {
        'version_number': 'versionNumber',
        'm_autotuning_in_progress': 'm_AutotuningInProgress',
        'm_percent_complete': 'm_PercentComplete',
        'm_autotuning_succeeded': 'm_AutotuningSucceeded',
        'm_num_patterns_to_autotune': 'm_NumPatternsToAutotune',
        'm_error_string_buffer': 'm_ErrorStringBuffer',
        'm_features_to_tune_array': 'm_FeaturesToTuneArray',
        'm_ncp': 'm_NCP',
        'm_ap': 'm_AP'
    }

    def __init__(self, version_number=None, m_autotuning_in_progress=None, m_percent_complete=None, m_autotuning_succeeded=None, m_num_patterns_to_autotune=None, m_error_string_buffer=None, m_features_to_tune_array=None, m_ncp=None, m_ap=None):
        """MAutotune - a model defined in Swagger"""
        self._version_number = None
        self._m_autotuning_in_progress = None
        self._m_percent_complete = None
        self._m_autotuning_succeeded = None
        self._m_num_patterns_to_autotune = None
        self._m_error_string_buffer = None
        self._m_features_to_tune_array = None
        self._m_ncp = None
        self._m_ap = None
        self.discriminator = None
        if version_number is not None:
            self.version_number = version_number
        if m_autotuning_in_progress is not None:
            self.m_autotuning_in_progress = m_autotuning_in_progress
        if m_percent_complete is not None:
            self.m_percent_complete = m_percent_complete
        if m_autotuning_succeeded is not None:
            self.m_autotuning_succeeded = m_autotuning_succeeded
        if m_num_patterns_to_autotune is not None:
            self.m_num_patterns_to_autotune = m_num_patterns_to_autotune
        if m_error_string_buffer is not None:
            self.m_error_string_buffer = m_error_string_buffer
        if m_features_to_tune_array is not None:
            self.m_features_to_tune_array = m_features_to_tune_array
        if m_ncp is not None:
            self.m_ncp = m_ncp
        self.m_ap = m_ap

    @property
    def version_number(self):
        """Gets the version_number of this MAutotune.


        :return: The version_number of this MAutotune.
        :rtype: VersionNumber
        """
        return self._version_number

    @version_number.setter
    def version_number(self, version_number):
        """Sets the version_number of this MAutotune.


        :param version_number: The version_number of this MAutotune.
        :type: VersionNumber
        """

        self._version_number = version_number

    @property
    def m_autotuning_in_progress(self):
        """Gets the m_autotuning_in_progress of this MAutotune.


        :return: The m_autotuning_in_progress of this MAutotune.
        :rtype: bool
        """
        return self._m_autotuning_in_progress

    @m_autotuning_in_progress.setter
    def m_autotuning_in_progress(self, m_autotuning_in_progress):
        """Sets the m_autotuning_in_progress of this MAutotune.


        :param m_autotuning_in_progress: The m_autotuning_in_progress of this MAutotune.
        :type: bool
        """

        self._m_autotuning_in_progress = m_autotuning_in_progress

    @property
    def m_percent_complete(self):
        """Gets the m_percent_complete of this MAutotune.


        :return: The m_percent_complete of this MAutotune.
        :rtype: float
        """
        return self._m_percent_complete

    @m_percent_complete.setter
    def m_percent_complete(self, m_percent_complete):
        """Sets the m_percent_complete of this MAutotune.


        :param m_percent_complete: The m_percent_complete of this MAutotune.
        :type: float
        """

        self._m_percent_complete = m_percent_complete

    @property
    def m_autotuning_succeeded(self):
        """Gets the m_autotuning_succeeded of this MAutotune.


        :return: The m_autotuning_succeeded of this MAutotune.
        :rtype: bool
        """
        return self._m_autotuning_succeeded

    @m_autotuning_succeeded.setter
    def m_autotuning_succeeded(self, m_autotuning_succeeded):
        """Sets the m_autotuning_succeeded of this MAutotune.


        :param m_autotuning_succeeded: The m_autotuning_succeeded of this MAutotune.
        :type: bool
        """

        self._m_autotuning_succeeded = m_autotuning_succeeded

    @property
    def m_num_patterns_to_autotune(self):
        """Gets the m_num_patterns_to_autotune of this MAutotune.


        :return: The m_num_patterns_to_autotune of this MAutotune.
        :rtype: float
        """
        return self._m_num_patterns_to_autotune

    @m_num_patterns_to_autotune.setter
    def m_num_patterns_to_autotune(self, m_num_patterns_to_autotune):
        """Sets the m_num_patterns_to_autotune of this MAutotune.


        :param m_num_patterns_to_autotune: The m_num_patterns_to_autotune of this MAutotune.
        :type: float
        """

        self._m_num_patterns_to_autotune = m_num_patterns_to_autotune

    @property
    def m_error_string_buffer(self):
        """Gets the m_error_string_buffer of this MAutotune.


        :return: The m_error_string_buffer of this MAutotune.
        :rtype: str
        """
        return self._m_error_string_buffer

    @m_error_string_buffer.setter
    def m_error_string_buffer(self, m_error_string_buffer):
        """Sets the m_error_string_buffer of this MAutotune.


        :param m_error_string_buffer: The m_error_string_buffer of this MAutotune.
        :type: str
        """

        self._m_error_string_buffer = m_error_string_buffer

    @property
    def m_features_to_tune_array(self):
        """Gets the m_features_to_tune_array of this MAutotune.


        :return: The m_features_to_tune_array of this MAutotune.
        :rtype: list[bool]
        """
        return self._m_features_to_tune_array

    @m_features_to_tune_array.setter
    def m_features_to_tune_array(self, m_features_to_tune_array):
        """Sets the m_features_to_tune_array of this MAutotune.


        :param m_features_to_tune_array: The m_features_to_tune_array of this MAutotune.
        :type: list[bool]
        """

        self._m_features_to_tune_array = m_features_to_tune_array

    @property
    def m_ncp(self):
        """Gets the m_ncp of this MAutotune.


        :return: The m_ncp of this MAutotune.
        :rtype: MNCP
        """
        return self._m_ncp

    @m_ncp.setter
    def m_ncp(self, m_ncp):
        """Sets the m_ncp of this MAutotune.


        :param m_ncp: The m_ncp of this MAutotune.
        :type: MNCP
        """

        self._m_ncp = m_ncp

    @property
    def m_ap(self):
        """Gets the m_ap of this MAutotune.


        :return: The m_ap of this MAutotune.
        :rtype: MAP
        """
        return self._m_ap

    @m_ap.setter
    def m_ap(self, m_ap):
        """Sets the m_ap of this MAutotune.


        :param m_ap: The m_ap of this MAutotune.
        :type: MAP
        """
        if m_ap is None:
            raise ValueError("Invalid value for `m_ap`, must not be `None`")

        self._m_ap = m_ap

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(MAutotune, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MAutotune):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import os
import time
import json
import numpy as np
import base64
from functools import wraps
from .api_client import ApiClient
from .configuration import Configuration
from .rest import ApiException
from .api.default_api import DefaultApi
from .models import *


class AmberV2Client:
    def __init__(
        self,
        license_id: str = None,
        license_file: str = "~/.Amber.license",
        verify: bool = False,
        timeout: int = 300,
    ):
        """Main client which interfaces with the Amber cloud. Amber account
        credentials are discovered within a .Amber.license file located in the
        home directory, or optionally overridden using environment variables.

        Args:
            license_id: (type: str) license identifier label found within .Amber.license file
            license_file: (type: str) path to .Amber.license file
            verify: (type: boolean) Boolean, controls whether we verify the server’s TLS certificate
            timeout: (type: int) number of seconds to allow API call to wait

        Environment:

            `AMBER_V2_LICENSE_FILE`: sets license_file path

            `AMBER_V2_LICENSE_ID`: sets license_id

            `AMBER_V2_SERVER`: overrides the server as found in .Amber.license file

            `AMBER_V2_OAUTH_SERVER`: overrides the oauth server as found in .Amber.license file

            `AMBER_V2_LICENSE_KEY`: overrides the license key as found in .Amber.license file

            `AMBER_V2_SECRET_KEY`: overrides the secret key as found in .Amber.license file

        Raises:
            ApiException: if error supplying authentication credentials
        """
        self.access_token = ""
        self.refresh_token = ""
        self.reauth_time = 0
        self.server = ""
        self.oauth_server = ""
        self.profile = os.environ.get("AMBER_V2_LICENSE_FILE", license_file)
        self.profile_id = os.environ.get("AMBER_V2_LICENSE_ID", license_id)
        self.api = None
        self.license = ""
        self.secret = ""
        file_data = ""

        if self.profile is not None:
            self.profile = os.path.expanduser(self.profile)
            if os.path.exists(self.profile):
                try:
                    with open(self.profile, "r") as f:
                        file_data = json.load(f)
                except json.JSONDecodeError as e:
                    raise ApiException("JSON formatting error in license file: {}, line: {}, col: {}".format(e.msg, e.lineno, e.colno))
                try:
                    self.server = file_data[self.profile_id]["server"]
                except KeyError:
                    # raise ApiException('profile_id "{} server" not found in license file'.format(self.profile_id))
                    # server not found but continue incase the env variable is set instead
                    self.server = ""
                try:
                    self.oauth_server = file_data[self.profile_id]["oauth-server"]
                except KeyError:
                    # raise ApiException('profile_id "{} oauth-server" not found in license file'.format(self.profile_id))
                    # server not found but continue incase the env variable is set instead
                    self.oauth_server = ""
            else:
                raise ApiException('Amber license file "{}" not found'.format(self.profile))

        self.configuration = Configuration()

        self.server = os.environ.get("AMBER_V2_SERVER", self.server)
        if self.server == "":
            raise ApiException('server not set: add "server" key to license file or set AMBER_V2_SERVER environment variable')
        # server is set if it reaches this point
        self.configuration.host = self.server

        # oauth server
        self.oauth_server = os.environ.get("AMBER_V2_OAUTH_SERVER", self.oauth_server)
        if self.oauth_server == "":
            # oauth_server gets assigned to server when not directly configured
            self.oauth_server = self.server

        try:
            self.license = file_data[self.profile_id]["license"]
            self.secret = file_data[self.profile_id]["secret"]
        except KeyError as e:
            # do nothing here
            self.license = ""
            self.secret = ""

        self.license = os.environ.get("AMBER_V2_LICENSE_KEY", self.license)
        self.secret = os.environ.get("AMBER_V2_SECRET_KEY", self.secret)

        if self.license == "" or self.secret == "":
            raise ApiException("need license and secret keys set in either the environment or the .Amber.license file")

        self.api = DefaultApi(ApiClient(self.configuration))

    def __authenticate(f):
        @wraps(f)
        def inner(self, *args, **kwargs):
            if time.time() <= self.reauth_time:
                return f(self, *args, **kwargs)

            try:
                if self.access_token == "":
                    # initial authentication, use license and secret key
                    body = PostOauth2AccessRequest(self.license, self.secret)
                    response = self.api.post_oauth2_access(body)
                    self.access_token = response.id_token
                    self.refresh_token = response.refresh_token
                    self.expires_in = int(response.expires_in)
                    self.secret = ""  # clear the secret from plain site
                else:
                    # we have authenticated once, use the refresh token
                    body = PostOauth2RefreshRequest(self.refresh_token)
                    response = self.api.post_oauth2_refresh(body)
                    self.access_token = response.id_token
                    self.refresh_token = response.refresh_token
                    self.expires_in = int(response.expires_in)

            except ApiException as e:
                raise ApiException("Authentication failed: invalid credentials")

            self.configuration.api_key["Authorization"] = self.access_token
            self.configuration.api_key_prefix["Authorization"] = "Bearer"
            self.reauth_time = time.time() + self.expires_in - 60

            return f(self, *args, **kwargs)

        return inner

    @__authenticate
    def get_version(self) -> GetVersionResponse:
        """

        Return version information for the API.

        Returns:
            `boonamber.v2.models.get_version_response.GetVersionResponse`

        Example:
            ```
            amber = AmberClientV2()
            version = amber.get_version()
            print(version.to_dict())
            ```

        """
        try:
            # get version
            return self.api.get_version()
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def delete_model(self, model_id: str):
        """

        Permanently delete the specified model.

        Args:
            model_id: (type: str) (required)

        """
        try:
            # delete model
            self.api.delete_model(model_id=model_id)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def get_root_cause(self, model_id: str, **kwargs) -> GetRootCauseResponse:
        """

        Return a measure of the significance of each feature in the creation of a cluster. The values range from 0 to 1 where a relatively high value represents a feature that was influential in creating the new cluster. No conclusions can be drawn from values close to zero. This measure can be computed for existing clusters or for individual vectors directly.

        Args:
            model_id: (type: str) (required)
            clusters: (type: str or array-like) Clusters to analyze (list of comma-separated integers).
            vectors: (type: str or array-like) Vectors to analyze, as a flat list of comma-separated floats. Number of values must be a multiple of the configured number of features.

        Returns:
            `boonamber.v2.models.get_root_cause_response.GetRootCauseResponse`

        """
        import numpy as np

        # vectors
        if "vectors" in kwargs.keys():
            if isinstance(kwargs["vectors"], (list, np.ndarray, tuple)):
                dimensions = len(np.asarray(kwargs["vectors"]).shape)
                # 1 vector given
                if dimensions == 1:
                    kwargs["vectors"] = ",".join([str(v) for v in kwargs["vectors"]])
                # 2d array of vectors
                elif dimensions == 2:
                    kwargs["vectors"] = [",".join([str(v) for v in row]) for row in kwargs["vectors"]]
                    kwargs["vectors"] = "],[".join(kwargs["vectors"])
                else:
                    raise ApiException("401: invalid dimensions of vectors given: should be 1 or 2D but got {}D".format(len(np.asarray(kwargs["vectors"]).shape)))
                kwargs["vectors"] = "[[{}]]".format(kwargs["vectors"])
            # not a string or not formatted as a string list
            elif not isinstance(kwargs["vectors"], str):
                raise ApiException("401: invalid formatting of vectors. Expecting a array-type or numbers or string")
        # clusters
        if "clusters" in kwargs.keys():
            if isinstance(kwargs["clusters"], (list, np.ndarray, tuple, str, int, float)):
                dimensions = len(np.asarray(kwargs["clusters"]).shape)
                # 1 value
                if dimensions == 0:
                    kwargs["clusters"] = f"[{kwargs['clusters']}]"
                # list of clusters
                elif dimensions == 1:
                    kwargs["clusters"] = "[{}]".format(",".join([str(c) for c in kwargs["clusters"]]))
                else:
                    raise ApiException("401: invalid dimensions of clusters given: should be 0 or 1D but got {}D".format(len(np.asarray(kwargs["clusters"]).shape)))
            else:
                raise ApiException("401: invalid formatting of clusters. Expecting a array-type or numbers or string")

        try:
            # get root cause
            if "clusters" in kwargs.keys():
                return self.api.get_model_root_cause(model_id=model_id, clusters=kwargs["clusters"])
            if "vectors" in kwargs.keys():
                return self.api.get_model_root_cause(model_id=model_id, vectors=kwargs["vectors"])
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def get_config(self, model_id: str) -> PostConfigResponse:
        """

        Get the configuration of the specified model.

        Args:
            model_id: (type: str) (required)

        Returns:
            `boonamber.v2.models.post_config_response.PostConfigResponse`

        """
        try:
            # get config
            return self.api.get_model_config(model_id=model_id)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def get_model(self, model_id: str) -> PostModelResponse:
        """

        Return metadata for the specified model.

        Args:
            model_id: (type: str) (required)

        Returns:
            `boonamber.v2.models.post_model_response.PostModelResponse`

        """
        try:
            # get model
            return self.api.get_model(model_id=model_id)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def get_models(self) -> GetModelsResponse:
        """

        Return `id` and `label` for all models belonging to the user.

        Returns:
            `boonamber.v2.models.get_models_response.GetModelsResponse`

        """
        try:
            # get list of models
            return self.api.get_models()
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def get_pretrain(self, model_id: str) -> GetPretrainResponse:
        """

        Get the pretraining status of the specified model.

        Args:
            model_id: (type: str) (required)

        Returns:
            `boonamber.v2.models.get_pretrain_response.GetPretrainResponse`

        """
        try:
            # get pretrain state
            return self.api.get_model_pretrain(model_id=model_id)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def get_status(self, model_id: str) -> GetStatusResponse:
        """

        Get the current state and learning progress of the specified model.

        Args:
            model_id: (type: str) (required)

        Returns:
            `boonamber.v2.models.get_status_response.GetStatusResponse`

        """
        try:
            # get status
            return self.api.get_model_status(model_id=model_id)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def get_nano_status(self, model_id: str) -> GetNanoStatusResponse:
        """

        Get the current nano status of the specified model.

        Args:
            model_id: (type: str) (required)

        Returns:
            `boonamber.v2.models.get_nano_status_response.GetNanoStatusResponse`

        """
        try:
            # get status
            return self.api.get_model_nano_status(model_id=model_id)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def get_diagnostic(self, model_id: str, dir: str) -> str:
        """

        Get the current summation of the specified model

        Args:
            model_id: (type: str) (required)
            dir: (type: str) (required) path to save the diagnostic tar file

        Returns:
            str

        """
        import os

        if not os.path.exists(dir):
            raise ApiException("target directory does not exist")
        dir = os.path.expanduser(dir)
        path = f"{dir}/{model_id}-diagnostic.tar"
        try:
            # get diagnostic
            results = self.api.get_model_diagnostic(model_id=model_id)

            with open(path, "wb") as fp:
                fp.write(results)
            return path
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def post_config(self, model_id: str, body: PostConfigRequest) -> PostConfigResponse:
        """

        Configure the specified model. Wipes all progress and puts the model in the `Buffering` state.

        Args:
            model_id: (type: str) (required)
            body: (type: `boonamber.v2.models.post_config_request.PostConfigRequest`) configuration to apply

        Returns:
            `boonamber.v2.models.post_config_response.PostConfigResponse`

        """

        try:
            # post config
            return self.api.post_model_config(model_id=model_id, body=body)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def post_data(self, model_id: str, data, save_image: bool = True) -> PostDataResponse:
        """

        Send data to the specified model, and get back the resulting analytics and model status.

        Args:
            model_id: (type: str) (required)
            data: (type: str or array-like) (required) data vector or vectors as a flattened list of comma-separated values
            save_image: (type: boolean) whether or not to save the model (only applies to on prem)

        Returns:
            `boonamber.v2.models.post_data_response.PostDataResponse`

        """

        try:
            # accept data as a list
            if isinstance(data, (list, np.ndarray)):
                data = np.array(data, dtype=str).flatten()
                data = ",".join(data)
            elif not isinstance(data, str):
                raise ApiException("invalid data: {}".format(type(data)))
            # post data
            body = PostDataRequest(data=data, save_image=save_image)
            return self.api.post_model_data(model_id=model_id, body=body)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def post_model(self, metadata: PostModelRequest) -> PostModelResponse:
        """

        Create a new model and return its unique identifier.

        Args:
            metadata: (type: `boonamber.v2.models.post_model_request.PostModelRequest`) (required) initial metadata for new model

        Returns:
            `boonamber.v2.models.post_model_response.PostModelResponse`

        """
        try:
            # post new model
            return self.api.post_model(body=metadata)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def copy_model(self, model_id: str, label: str = None) -> PostModelResponse:
        """

        Copy a model and return the new model information.

        Args:
            model_id: (type: str) (required)
            label: (type: str) label for new model (uses previous label if unspecified)

        Returns:
            `boonamber.v2.models.post_model_response.PostModelResponse`

        """
        kwargs = {}
        if label:
            kwargs["body"] = PostModelCopyRequest(label=label)

        try:
            # copy a model
            return self.api.post_model_copy(model_id, **kwargs)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def post_outage(self, model_id: str):
        """

        Resets the streaming window generated by `streamingWindow`. This endpoint should be called after a data outage before resuming streaming.

        Args:
            model_id: (type: str) (required)

        """
        try:
            # post outage
            self.api.post_model_outage(model_id=model_id)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def migrate_model(self, v1_model_id: str):
        """migrate a v1 sensor to a v2 model


        Args:
            v1_model_id: (type: str) version 1 sensor id (required)

        Returns:
            `boonamber.v2.models.post_model_response.PostModelResponse`

        """

        try:
            # post migrate
            return self.api.post_model_migrate(v1_model_id=v1_model_id)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def post_pretrain(self, model_id: str, data, chunk_size: int = 400000, block: bool = True, **kwargs) -> PostPretrainResponse:
        """pretrain model with an existing dataset

        Args:
            model_id: (type: str) (required)
            data: (type: str or array like) data to process
            chunk_size: (type: int) number of portions to send the data over
            block: (type: boolean) wait until pretraining finishes before returning

        Returns:
            `boonamber.v2.models.post_pretrain_response.PostPretrainResponse`

        """
        try:
            # Server expects data as a plaintext string of comma-separated values.
            try:
                if isinstance(data, str):
                    data = data.split(",")

                data = np.array(data, dtype="float32")
                data = data.flatten()
                data = data.tobytes()
            except ValueError as e:
                raise ApiException("invalid data: {}".format(e))

            # headers = {"content-type": "application/octet-stream"
            param = PostPretrainRequest(data="", format="packed-float")

            # compute number of chunks to send
            num_chunks = int(len(data) / chunk_size)
            if len(data) % chunk_size != 0:
                num_chunks += 1

            txn_id = ""
            for chunk_num in range(0, num_chunks):
                # create chunk specifier, .ie 1:3, 2:3, 3:3
                chunkspec = "{}:{}".format(chunk_num + 1, num_chunks)

                # construct next chunk
                start = chunk_num * chunk_size
                end = start + chunk_size
                if end > len(data):
                    end = len(data)
                param.data = base64.b64encode(data[start:end]).decode("ascii")

                try:
                    response = self.api.post_model_pretrain(
                        model_id=model_id,
                        chunkspec=chunkspec,
                        txn_id=txn_id,
                        body=param,
                    )
                    txn_id = response.txn_id
                except Exception as e:
                    raise (ApiException(e))

            if not block:
                return response

            while response.status == "Pretraining":
                time.sleep(3)
                response = self.get_pretrain(model_id=model_id)

            return response

        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def enable_learning(self, model_id: str, **kwargs) -> PostLearningResponse:
        """

        Update model configuration and re-enable learning

        Args:
            model_id: (type: str) (required)
            training: (type: `boonamber.v2.models.training_config.TrainingConfig`) updates to apply

        Returns:
            `boonamber.v2.models.post_learning_response.PostLearningResponse`

        """
        if "training" in kwargs:
            kwargs["body"] = PostLearningRequest(training=kwargs["training"])
            del kwargs["training"]
        try:
            # start learning again
            return self.api.post_model_learning(model_id=model_id, **kwargs)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def put_data(self, model_id: str, body: PutDataRequest) -> PutDataResponse:
        """update fusion vector and get back results

        Args:
            model_id: (type: str) (required)
            body: (type: `boonamber.v2.models.put_data_request.PutDataRequest`) (required) updates to the fusion vector

        Returns:
            `boonamber.v2.models.put_data_response.PutDataResponse`

        """
        try:
            # put data
            return self.api.put_model_data(model_id=model_id, body=body)
        except Exception as e:
            raise ApiException(e)

    @__authenticate
    def update_label(self, model_id: str, label: str) -> PostModelResponse:
        """

        Update metadata for the specified model.

        Args:
            model_id: (type: str) (required)
            label: (type: str) (required) updates to apply

        Returns:
            `boonamber.v2.models.post_model_response.PostModelResponse`

        """
        try:
            # update model metadata
            metadata = PutModelRequest(label=label)
            return self.api.put_model(model_id=model_id, body=metadata)
        except Exception as e:
            raise ApiException(e)



# coding: utf-8

"""
    Amber API Server

    Boon Logic Amber API server

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PostPretrainRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'data': 'str',
        'format': 'str'
    }

    attribute_map = {
        'data': 'data',
        'format': 'format'
    }

    def __init__(self, data=None, format='csv'):
        """PostPretrainRequest - a model defined in Swagger"""
        self._data = None
        self._format = None
        self.discriminator = None
        self.data = data
        if format is not None:
            self.format = format

    @property
    def data(self):
        """Gets the data of this PostPretrainRequest.

        Data in one of two formats: 1) A flat list of comma-separated values. 2) The string that results from flattening the dataset, packing the values into a byte buffer as float32s (little-endian), and base-64 encoding the buffer.  Datasets which are too large to send in one request may be sent in multiple chunks using the header parameters for chunked uploads (`txnId` and `chunkspec`).  The total number of data values sent for pretraining must be a multiple of the number of features in the configuration.

        :return: The data of this PostPretrainRequest.
        :rtype: str
        """
        return self._data

    @data.setter
    def data(self, data):
        """Sets the data of this PostPretrainRequest.

        Data in one of two formats: 1) A flat list of comma-separated values. 2) The string that results from flattening the dataset, packing the values into a byte buffer as float32s (little-endian), and base-64 encoding the buffer.  Datasets which are too large to send in one request may be sent in multiple chunks using the header parameters for chunked uploads (`txnId` and `chunkspec`).  The total number of data values sent for pretraining must be a multiple of the number of features in the configuration.

        :param data: The data of this PostPretrainRequest.
        :type: str
        """
        if data is None:
            raise ValueError("Invalid value for `data`, must not be `None`")

        self._data = data

    @property
    def format(self):
        """Gets the format of this PostPretrainRequest.

        Format specifier for `data`.

        :return: The format of this PostPretrainRequest.
        :rtype: str
        """
        return self._format

    @format.setter
    def format(self, format):
        """Sets the format of this PostPretrainRequest.

        Format specifier for `data`.

        :param format: The format of this PostPretrainRequest.
        :type: str
        """
        allowed_values = ["csv", "b64float", "packed-float"]
        if format not in allowed_values:
            raise ValueError(
                "Invalid value for `format` ({0}), must be one of {1}"
                .format(format, allowed_values)
            )

        self._format = format

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PostPretrainRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PostPretrainRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Amber API Server

    Boon Logic Amber API server

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PostDataRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'data': 'str',
        'save_image': 'bool'
    }

    attribute_map = {
        'data': 'data',
        'save_image': 'saveImage'
    }

    def __init__(self, data=None, save_image=True):
        """PostDataRequest - a model defined in Swagger"""
        self._data = None
        self._save_image = None
        self.discriminator = None
        self.data = data
        if save_image is not None:
            self.save_image = save_image

    @property
    def data(self):
        """Gets the data of this PostDataRequest.

        Data as a flat list of comma-separated values. The number of values sent must be a multiple of the number of features in the configuration.

        :return: The data of this PostDataRequest.
        :rtype: str
        """
        return self._data

    @data.setter
    def data(self, data):
        """Sets the data of this PostDataRequest.

        Data as a flat list of comma-separated values. The number of values sent must be a multiple of the number of features in the configuration.

        :param data: The data of this PostDataRequest.
        :type: str
        """
        if data is None:
            raise ValueError("Invalid value for `data`, must not be `None`")

        self._data = data

    @property
    def save_image(self):
        """Gets the save_image of this PostDataRequest.

        If false, saved model image will not be updated on this request. Applies only to on-premises deployments.

        :return: The save_image of this PostDataRequest.
        :rtype: bool
        """
        return self._save_image

    @save_image.setter
    def save_image(self, save_image):
        """Sets the save_image of this PostDataRequest.

        If false, saved model image will not be updated on this request. Applies only to on-premises deployments.

        :param save_image: The save_image of this PostDataRequest.
        :type: bool
        """

        self._save_image = save_image

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PostDataRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PostDataRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



class AmberUserError(Exception):
    """Raised to indicate an error in SDK usage"""

    def __init__(self, message):
        self.message = message


class AmberCloudError(Exception):
    """Raised upon any non-200 response from the Amber cloud"""

    def __init__(self, code, message):
        self.code = code
        self.message = message
        super().__init__("{}: {}".format(code, message))



# coding: utf-8

"""
    Amber API Server

    Boon Logic Amber API server

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class NanoStatus(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'pca': 'list[list[float]]',
        'cluster_growth': 'list[float]',
        'cluster_sizes': 'list[float]',
        'anomaly_indexes': 'list[float]',
        'frequency_indexes': 'list[float]',
        'distance_indexes': 'list[float]',
        'sample_count': 'int',
        'cluster_count': 'int',
        'anomaly_threshold': 'int',
        'state': 'AmberState'
    }

    attribute_map = {
        'pca': 'pca',
        'cluster_growth': 'clusterGrowth',
        'cluster_sizes': 'clusterSizes',
        'anomaly_indexes': 'anomalyIndexes',
        'frequency_indexes': 'frequencyIndexes',
        'distance_indexes': 'distanceIndexes',
        'sample_count': 'sampleCount',
        'cluster_count': 'clusterCount',
        'anomaly_threshold': 'anomalyThreshold',
        'state': 'state'
    }

    def __init__(self, pca=None, cluster_growth=None, cluster_sizes=None, anomaly_indexes=None, frequency_indexes=None, distance_indexes=None, sample_count=None, cluster_count=None, anomaly_threshold=None, state=None):
        """NanoStatus - a model defined in Swagger"""
        self._pca = None
        self._cluster_growth = None
        self._cluster_sizes = None
        self._anomaly_indexes = None
        self._frequency_indexes = None
        self._distance_indexes = None
        self._sample_count = None
        self._cluster_count = None
        self._anomaly_threshold = None
        self._state = None
        self.discriminator = None
        if pca is not None:
            self.pca = pca
        if cluster_growth is not None:
            self.cluster_growth = cluster_growth
        if cluster_sizes is not None:
            self.cluster_sizes = cluster_sizes
        if anomaly_indexes is not None:
            self.anomaly_indexes = anomaly_indexes
        if frequency_indexes is not None:
            self.frequency_indexes = frequency_indexes
        if distance_indexes is not None:
            self.distance_indexes = distance_indexes
        if sample_count is not None:
            self.sample_count = sample_count
        if cluster_count is not None:
            self.cluster_count = cluster_count
        if anomaly_threshold is not None:
            self.anomaly_threshold = anomaly_threshold
        if state is not None:
            self.state = state

    @property
    def pca(self):
        """Gets the pca of this NanoStatus.

        pca

        :return: The pca of this NanoStatus.
        :rtype: list[list[float]]
        """
        return self._pca

    @pca.setter
    def pca(self, pca):
        """Sets the pca of this NanoStatus.

        pca

        :param pca: The pca of this NanoStatus.
        :type: list[list[float]]
        """

        self._pca = pca

    @property
    def cluster_growth(self):
        """Gets the cluster_growth of this NanoStatus.

        cluster growth

        :return: The cluster_growth of this NanoStatus.
        :rtype: list[float]
        """
        return self._cluster_growth

    @cluster_growth.setter
    def cluster_growth(self, cluster_growth):
        """Sets the cluster_growth of this NanoStatus.

        cluster growth

        :param cluster_growth: The cluster_growth of this NanoStatus.
        :type: list[float]
        """

        self._cluster_growth = cluster_growth

    @property
    def cluster_sizes(self):
        """Gets the cluster_sizes of this NanoStatus.

        cluster sizes

        :return: The cluster_sizes of this NanoStatus.
        :rtype: list[float]
        """
        return self._cluster_sizes

    @cluster_sizes.setter
    def cluster_sizes(self, cluster_sizes):
        """Sets the cluster_sizes of this NanoStatus.

        cluster sizes

        :param cluster_sizes: The cluster_sizes of this NanoStatus.
        :type: list[float]
        """

        self._cluster_sizes = cluster_sizes

    @property
    def anomaly_indexes(self):
        """Gets the anomaly_indexes of this NanoStatus.

        anomaly indexes

        :return: The anomaly_indexes of this NanoStatus.
        :rtype: list[float]
        """
        return self._anomaly_indexes

    @anomaly_indexes.setter
    def anomaly_indexes(self, anomaly_indexes):
        """Sets the anomaly_indexes of this NanoStatus.

        anomaly indexes

        :param anomaly_indexes: The anomaly_indexes of this NanoStatus.
        :type: list[float]
        """

        self._anomaly_indexes = anomaly_indexes

    @property
    def frequency_indexes(self):
        """Gets the frequency_indexes of this NanoStatus.

        frequency indexes

        :return: The frequency_indexes of this NanoStatus.
        :rtype: list[float]
        """
        return self._frequency_indexes

    @frequency_indexes.setter
    def frequency_indexes(self, frequency_indexes):
        """Sets the frequency_indexes of this NanoStatus.

        frequency indexes

        :param frequency_indexes: The frequency_indexes of this NanoStatus.
        :type: list[float]
        """

        self._frequency_indexes = frequency_indexes

    @property
    def distance_indexes(self):
        """Gets the distance_indexes of this NanoStatus.

        distance indexes

        :return: The distance_indexes of this NanoStatus.
        :rtype: list[float]
        """
        return self._distance_indexes

    @distance_indexes.setter
    def distance_indexes(self, distance_indexes):
        """Sets the distance_indexes of this NanoStatus.

        distance indexes

        :param distance_indexes: The distance_indexes of this NanoStatus.
        :type: list[float]
        """

        self._distance_indexes = distance_indexes

    @property
    def sample_count(self):
        """Gets the sample_count of this NanoStatus.

        Total number of patterns processed since configuration.

        :return: The sample_count of this NanoStatus.
        :rtype: int
        """
        return self._sample_count

    @sample_count.setter
    def sample_count(self, sample_count):
        """Sets the sample_count of this NanoStatus.

        Total number of patterns processed since configuration.

        :param sample_count: The sample_count of this NanoStatus.
        :type: int
        """

        self._sample_count = sample_count

    @property
    def cluster_count(self):
        """Gets the cluster_count of this NanoStatus.

        Total number of clusters generated by patterns so far.

        :return: The cluster_count of this NanoStatus.
        :rtype: int
        """
        return self._cluster_count

    @cluster_count.setter
    def cluster_count(self, cluster_count):
        """Sets the cluster_count of this NanoStatus.

        Total number of clusters generated by patterns so far.

        :param cluster_count: The cluster_count of this NanoStatus.
        :type: int
        """

        self._cluster_count = cluster_count

    @property
    def anomaly_threshold(self):
        """Gets the anomaly_threshold of this NanoStatus.

        anomaly threshold

        :return: The anomaly_threshold of this NanoStatus.
        :rtype: int
        """
        return self._anomaly_threshold

    @anomaly_threshold.setter
    def anomaly_threshold(self, anomaly_threshold):
        """Sets the anomaly_threshold of this NanoStatus.

        anomaly threshold

        :param anomaly_threshold: The anomaly_threshold of this NanoStatus.
        :type: int
        """

        self._anomaly_threshold = anomaly_threshold

    @property
    def state(self):
        """Gets the state of this NanoStatus.


        :return: The state of this NanoStatus.
        :rtype: AmberState
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this NanoStatus.


        :param state: The state of this NanoStatus.
        :type: AmberState
        """

        self._state = state

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(NanoStatus, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NanoStatus):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Amber API Server

    Boon Logic Amber API server

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class MAmberStatus(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'version_number': 'VersionNumber',
        'm_status': 'str',
        'm_message': 'str',
        'm_progress': 'float',
        'm_current_cluster_count': 'float',
        'm_retry_count': 'float',
        'm_streaming_window_size': 'float',
        'm_total_inferences': 'float',
        'm_total_samples_written': 'float'
    }

    attribute_map = {
        'version_number': 'versionNumber',
        'm_status': 'm_Status',
        'm_message': 'm_Message',
        'm_progress': 'm_Progress',
        'm_current_cluster_count': 'm_CurrentClusterCount',
        'm_retry_count': 'm_RetryCount',
        'm_streaming_window_size': 'm_StreamingWindowSize',
        'm_total_inferences': 'm_TotalInferences',
        'm_total_samples_written': 'm_TotalSamplesWritten'
    }

    def __init__(self, version_number=None, m_status=None, m_message=None, m_progress=None, m_current_cluster_count=None, m_retry_count=None, m_streaming_window_size=None, m_total_inferences=None, m_total_samples_written=None):
        """MAmberStatus - a model defined in Swagger"""
        self._version_number = None
        self._m_status = None
        self._m_message = None
        self._m_progress = None
        self._m_current_cluster_count = None
        self._m_retry_count = None
        self._m_streaming_window_size = None
        self._m_total_inferences = None
        self._m_total_samples_written = None
        self.discriminator = None
        if version_number is not None:
            self.version_number = version_number
        self.m_status = m_status
        if m_message is not None:
            self.m_message = m_message
        if m_progress is not None:
            self.m_progress = m_progress
        self.m_current_cluster_count = m_current_cluster_count
        self.m_retry_count = m_retry_count
        self.m_streaming_window_size = m_streaming_window_size
        self.m_total_inferences = m_total_inferences
        if m_total_samples_written is not None:
            self.m_total_samples_written = m_total_samples_written

    @property
    def version_number(self):
        """Gets the version_number of this MAmberStatus.


        :return: The version_number of this MAmberStatus.
        :rtype: VersionNumber
        """
        return self._version_number

    @version_number.setter
    def version_number(self, version_number):
        """Sets the version_number of this MAmberStatus.


        :param version_number: The version_number of this MAmberStatus.
        :type: VersionNumber
        """

        self._version_number = version_number

    @property
    def m_status(self):
        """Gets the m_status of this MAmberStatus.


        :return: The m_status of this MAmberStatus.
        :rtype: str
        """
        return self._m_status

    @m_status.setter
    def m_status(self, m_status):
        """Sets the m_status of this MAmberStatus.


        :param m_status: The m_status of this MAmberStatus.
        :type: str
        """
        if m_status is None:
            raise ValueError("Invalid value for `m_status`, must not be `None`")

        self._m_status = m_status

    @property
    def m_message(self):
        """Gets the m_message of this MAmberStatus.


        :return: The m_message of this MAmberStatus.
        :rtype: str
        """
        return self._m_message

    @m_message.setter
    def m_message(self, m_message):
        """Sets the m_message of this MAmberStatus.


        :param m_message: The m_message of this MAmberStatus.
        :type: str
        """

        self._m_message = m_message

    @property
    def m_progress(self):
        """Gets the m_progress of this MAmberStatus.


        :return: The m_progress of this MAmberStatus.
        :rtype: float
        """
        return self._m_progress

    @m_progress.setter
    def m_progress(self, m_progress):
        """Sets the m_progress of this MAmberStatus.


        :param m_progress: The m_progress of this MAmberStatus.
        :type: float
        """

        self._m_progress = m_progress

    @property
    def m_current_cluster_count(self):
        """Gets the m_current_cluster_count of this MAmberStatus.


        :return: The m_current_cluster_count of this MAmberStatus.
        :rtype: float
        """
        return self._m_current_cluster_count

    @m_current_cluster_count.setter
    def m_current_cluster_count(self, m_current_cluster_count):
        """Sets the m_current_cluster_count of this MAmberStatus.


        :param m_current_cluster_count: The m_current_cluster_count of this MAmberStatus.
        :type: float
        """
        if m_current_cluster_count is None:
            raise ValueError("Invalid value for `m_current_cluster_count`, must not be `None`")

        self._m_current_cluster_count = m_current_cluster_count

    @property
    def m_retry_count(self):
        """Gets the m_retry_count of this MAmberStatus.


        :return: The m_retry_count of this MAmberStatus.
        :rtype: float
        """
        return self._m_retry_count

    @m_retry_count.setter
    def m_retry_count(self, m_retry_count):
        """Sets the m_retry_count of this MAmberStatus.


        :param m_retry_count: The m_retry_count of this MAmberStatus.
        :type: float
        """
        if m_retry_count is None:
            raise ValueError("Invalid value for `m_retry_count`, must not be `None`")

        self._m_retry_count = m_retry_count

    @property
    def m_streaming_window_size(self):
        """Gets the m_streaming_window_size of this MAmberStatus.


        :return: The m_streaming_window_size of this MAmberStatus.
        :rtype: float
        """
        return self._m_streaming_window_size

    @m_streaming_window_size.setter
    def m_streaming_window_size(self, m_streaming_window_size):
        """Sets the m_streaming_window_size of this MAmberStatus.


        :param m_streaming_window_size: The m_streaming_window_size of this MAmberStatus.
        :type: float
        """
        if m_streaming_window_size is None:
            raise ValueError("Invalid value for `m_streaming_window_size`, must not be `None`")

        self._m_streaming_window_size = m_streaming_window_size

    @property
    def m_total_inferences(self):
        """Gets the m_total_inferences of this MAmberStatus.


        :return: The m_total_inferences of this MAmberStatus.
        :rtype: float
        """
        return self._m_total_inferences

    @m_total_inferences.setter
    def m_total_inferences(self, m_total_inferences):
        """Sets the m_total_inferences of this MAmberStatus.


        :param m_total_inferences: The m_total_inferences of this MAmberStatus.
        :type: float
        """
        if m_total_inferences is None:
            raise ValueError("Invalid value for `m_total_inferences`, must not be `None`")

        self._m_total_inferences = m_total_inferences

    @property
    def m_total_samples_written(self):
        """Gets the m_total_samples_written of this MAmberStatus.


        :return: The m_total_samples_written of this MAmberStatus.
        :rtype: float
        """
        return self._m_total_samples_written

    @m_total_samples_written.setter
    def m_total_samples_written(self, m_total_samples_written):
        """Sets the m_total_samples_written of this MAmberStatus.


        :param m_total_samples_written: The m_total_samples_written of this MAmberStatus.
        :type: float
        """

        self._m_total_samples_written = m_total_samples_written

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(MAmberStatus, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MAmberStatus):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Amber API Server

    Boon Logic Amber API server

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class MStreamingParameters(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'version_number': 'VersionNumber',
        'm_enable_autotuning': 'bool',
        'm_samples_to_buffer_for_autotuning': 'float',
        'm_graduation_at_rate_numerator': 'float',
        'm_graduation_at_rate_denominator': 'float',
        'm_graduate_at_max_clusters': 'float',
        'm_graduate_at_max_samples': 'float'
    }

    attribute_map = {
        'version_number': 'versionNumber',
        'm_enable_autotuning': 'm_EnableAutotuning',
        'm_samples_to_buffer_for_autotuning': 'm_SamplesToBufferForAutotuning',
        'm_graduation_at_rate_numerator': 'm_GraduationAtRateNumerator',
        'm_graduation_at_rate_denominator': 'm_GraduationAtRateDenominator',
        'm_graduate_at_max_clusters': 'm_GraduateAtMaxClusters',
        'm_graduate_at_max_samples': 'm_GraduateAtMaxSamples'
    }

    def __init__(self, version_number=None, m_enable_autotuning=None, m_samples_to_buffer_for_autotuning=None, m_graduation_at_rate_numerator=None, m_graduation_at_rate_denominator=None, m_graduate_at_max_clusters=None, m_graduate_at_max_samples=None):
        """MStreamingParameters - a model defined in Swagger"""
        self._version_number = None
        self._m_enable_autotuning = None
        self._m_samples_to_buffer_for_autotuning = None
        self._m_graduation_at_rate_numerator = None
        self._m_graduation_at_rate_denominator = None
        self._m_graduate_at_max_clusters = None
        self._m_graduate_at_max_samples = None
        self.discriminator = None
        if version_number is not None:
            self.version_number = version_number
        if m_enable_autotuning is not None:
            self.m_enable_autotuning = m_enable_autotuning
        self.m_samples_to_buffer_for_autotuning = m_samples_to_buffer_for_autotuning
        self.m_graduation_at_rate_numerator = m_graduation_at_rate_numerator
        self.m_graduation_at_rate_denominator = m_graduation_at_rate_denominator
        self.m_graduate_at_max_clusters = m_graduate_at_max_clusters
        self.m_graduate_at_max_samples = m_graduate_at_max_samples

    @property
    def version_number(self):
        """Gets the version_number of this MStreamingParameters.


        :return: The version_number of this MStreamingParameters.
        :rtype: VersionNumber
        """
        return self._version_number

    @version_number.setter
    def version_number(self, version_number):
        """Sets the version_number of this MStreamingParameters.


        :param version_number: The version_number of this MStreamingParameters.
        :type: VersionNumber
        """

        self._version_number = version_number

    @property
    def m_enable_autotuning(self):
        """Gets the m_enable_autotuning of this MStreamingParameters.


        :return: The m_enable_autotuning of this MStreamingParameters.
        :rtype: bool
        """
        return self._m_enable_autotuning

    @m_enable_autotuning.setter
    def m_enable_autotuning(self, m_enable_autotuning):
        """Sets the m_enable_autotuning of this MStreamingParameters.


        :param m_enable_autotuning: The m_enable_autotuning of this MStreamingParameters.
        :type: bool
        """

        self._m_enable_autotuning = m_enable_autotuning

    @property
    def m_samples_to_buffer_for_autotuning(self):
        """Gets the m_samples_to_buffer_for_autotuning of this MStreamingParameters.


        :return: The m_samples_to_buffer_for_autotuning of this MStreamingParameters.
        :rtype: float
        """
        return self._m_samples_to_buffer_for_autotuning

    @m_samples_to_buffer_for_autotuning.setter
    def m_samples_to_buffer_for_autotuning(self, m_samples_to_buffer_for_autotuning):
        """Sets the m_samples_to_buffer_for_autotuning of this MStreamingParameters.


        :param m_samples_to_buffer_for_autotuning: The m_samples_to_buffer_for_autotuning of this MStreamingParameters.
        :type: float
        """
        if m_samples_to_buffer_for_autotuning is None:
            raise ValueError("Invalid value for `m_samples_to_buffer_for_autotuning`, must not be `None`")

        self._m_samples_to_buffer_for_autotuning = m_samples_to_buffer_for_autotuning

    @property
    def m_graduation_at_rate_numerator(self):
        """Gets the m_graduation_at_rate_numerator of this MStreamingParameters.


        :return: The m_graduation_at_rate_numerator of this MStreamingParameters.
        :rtype: float
        """
        return self._m_graduation_at_rate_numerator

    @m_graduation_at_rate_numerator.setter
    def m_graduation_at_rate_numerator(self, m_graduation_at_rate_numerator):
        """Sets the m_graduation_at_rate_numerator of this MStreamingParameters.


        :param m_graduation_at_rate_numerator: The m_graduation_at_rate_numerator of this MStreamingParameters.
        :type: float
        """
        if m_graduation_at_rate_numerator is None:
            raise ValueError("Invalid value for `m_graduation_at_rate_numerator`, must not be `None`")

        self._m_graduation_at_rate_numerator = m_graduation_at_rate_numerator

    @property
    def m_graduation_at_rate_denominator(self):
        """Gets the m_graduation_at_rate_denominator of this MStreamingParameters.


        :return: The m_graduation_at_rate_denominator of this MStreamingParameters.
        :rtype: float
        """
        return self._m_graduation_at_rate_denominator

    @m_graduation_at_rate_denominator.setter
    def m_graduation_at_rate_denominator(self, m_graduation_at_rate_denominator):
        """Sets the m_graduation_at_rate_denominator of this MStreamingParameters.


        :param m_graduation_at_rate_denominator: The m_graduation_at_rate_denominator of this MStreamingParameters.
        :type: float
        """
        if m_graduation_at_rate_denominator is None:
            raise ValueError("Invalid value for `m_graduation_at_rate_denominator`, must not be `None`")

        self._m_graduation_at_rate_denominator = m_graduation_at_rate_denominator

    @property
    def m_graduate_at_max_clusters(self):
        """Gets the m_graduate_at_max_clusters of this MStreamingParameters.


        :return: The m_graduate_at_max_clusters of this MStreamingParameters.
        :rtype: float
        """
        return self._m_graduate_at_max_clusters

    @m_graduate_at_max_clusters.setter
    def m_graduate_at_max_clusters(self, m_graduate_at_max_clusters):
        """Sets the m_graduate_at_max_clusters of this MStreamingParameters.


        :param m_graduate_at_max_clusters: The m_graduate_at_max_clusters of this MStreamingParameters.
        :type: float
        """
        if m_graduate_at_max_clusters is None:
            raise ValueError("Invalid value for `m_graduate_at_max_clusters`, must not be `None`")

        self._m_graduate_at_max_clusters = m_graduate_at_max_clusters

    @property
    def m_graduate_at_max_samples(self):
        """Gets the m_graduate_at_max_samples of this MStreamingParameters.


        :return: The m_graduate_at_max_samples of this MStreamingParameters.
        :rtype: float
        """
        return self._m_graduate_at_max_samples

    @m_graduate_at_max_samples.setter
    def m_graduate_at_max_samples(self, m_graduate_at_max_samples):
        """Sets the m_graduate_at_max_samples of this MStreamingParameters.


        :param m_graduate_at_max_samples: The m_graduate_at_max_samples of this MStreamingParameters.
        :type: float
        """
        if m_graduate_at_max_samples is None:
            raise ValueError("Invalid value for `m_graduate_at_max_samples`, must not be `None`")

        self._m_graduate_at_max_samples = m_graduate_at_max_samples

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(MStreamingParameters, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MStreamingParameters):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Amber API Server

    Boon Logic Amber API server

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class TrainingConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'history_window': 'int',
        'buffering_samples': 'int',
        'learning_max_samples': 'int',
        'learning_max_clusters': 'int',
        'learning_rate_numerator': 'int',
        'learning_rate_denominator': 'int'
    }

    attribute_map = {
        'history_window': 'historyWindow',
        'buffering_samples': 'bufferingSamples',
        'learning_max_samples': 'learningMaxSamples',
        'learning_max_clusters': 'learningMaxClusters',
        'learning_rate_numerator': 'learningRateNumerator',
        'learning_rate_denominator': 'learningRateDenominator'
    }

    def __init__(self, history_window=None, buffering_samples=None, learning_max_samples=None, learning_max_clusters=None, learning_rate_numerator=None, learning_rate_denominator=None):
        """TrainingConfig - a model defined in Swagger"""
        self._history_window = None
        self._buffering_samples = None
        self._learning_max_samples = None
        self._learning_max_clusters = None
        self._learning_rate_numerator = None
        self._learning_rate_denominator = None
        self.discriminator = None
        if history_window is not None:
            self.history_window = history_window
        if buffering_samples is not None:
            self.buffering_samples = buffering_samples
        if learning_max_samples is not None:
            self.learning_max_samples = learning_max_samples
        if learning_max_clusters is not None:
            self.learning_max_clusters = learning_max_clusters
        if learning_rate_numerator is not None:
            self.learning_rate_numerator = learning_rate_numerator
        if learning_rate_denominator is not None:
            self.learning_rate_denominator = learning_rate_denominator

    @property
    def history_window(self):
        """Gets the history_window of this TrainingConfig.

        Number of past inferences to take into account when computing `warningLevel` at a given moment.

        :return: The history_window of this TrainingConfig.
        :rtype: int
        """
        return self._history_window

    @history_window.setter
    def history_window(self, history_window):
        """Sets the history_window of this TrainingConfig.

        Number of past inferences to take into account when computing `warningLevel` at a given moment.

        :param history_window: The history_window of this TrainingConfig.
        :type: int
        """

        self._history_window = history_window

    @property
    def buffering_samples(self):
        """Gets the buffering_samples of this TrainingConfig.

        Number of data vectors to collect during `Buffering`. These samples are used as data for `Autotuning`.

        :return: The buffering_samples of this TrainingConfig.
        :rtype: int
        """
        return self._buffering_samples

    @buffering_samples.setter
    def buffering_samples(self, buffering_samples):
        """Sets the buffering_samples of this TrainingConfig.

        Number of data vectors to collect during `Buffering`. These samples are used as data for `Autotuning`.

        :param buffering_samples: The buffering_samples of this TrainingConfig.
        :type: int
        """

        self._buffering_samples = buffering_samples

    @property
    def learning_max_samples(self):
        """Gets the learning_max_samples of this TrainingConfig.

        Maximum number of vectors to process during `Learning` before transitioning to `Monitoring`.

        :return: The learning_max_samples of this TrainingConfig.
        :rtype: int
        """
        return self._learning_max_samples

    @learning_max_samples.setter
    def learning_max_samples(self, learning_max_samples):
        """Sets the learning_max_samples of this TrainingConfig.

        Maximum number of vectors to process during `Learning` before transitioning to `Monitoring`.

        :param learning_max_samples: The learning_max_samples of this TrainingConfig.
        :type: int
        """

        self._learning_max_samples = learning_max_samples

    @property
    def learning_max_clusters(self):
        """Gets the learning_max_clusters of this TrainingConfig.

        Maximum number of clusters before model transitions from `Learning` to `Monitoring`.

        :return: The learning_max_clusters of this TrainingConfig.
        :rtype: int
        """
        return self._learning_max_clusters

    @learning_max_clusters.setter
    def learning_max_clusters(self, learning_max_clusters):
        """Sets the learning_max_clusters of this TrainingConfig.

        Maximum number of clusters before model transitions from `Learning` to `Monitoring`.

        :param learning_max_clusters: The learning_max_clusters of this TrainingConfig.
        :type: int
        """

        self._learning_max_clusters = learning_max_clusters

    @property
    def learning_rate_numerator(self):
        """Gets the learning_rate_numerator of this TrainingConfig.

        Switch to `Monitoring` if there were fewer than `learningRateNumerator` new clusters in the last `learningRateDenominator` inferences.

        :return: The learning_rate_numerator of this TrainingConfig.
        :rtype: int
        """
        return self._learning_rate_numerator

    @learning_rate_numerator.setter
    def learning_rate_numerator(self, learning_rate_numerator):
        """Sets the learning_rate_numerator of this TrainingConfig.

        Switch to `Monitoring` if there were fewer than `learningRateNumerator` new clusters in the last `learningRateDenominator` inferences.

        :param learning_rate_numerator: The learning_rate_numerator of this TrainingConfig.
        :type: int
        """

        self._learning_rate_numerator = learning_rate_numerator

    @property
    def learning_rate_denominator(self):
        """Gets the learning_rate_denominator of this TrainingConfig.

        See `learningRateNumerator`.

        :return: The learning_rate_denominator of this TrainingConfig.
        :rtype: int
        """
        return self._learning_rate_denominator

    @learning_rate_denominator.setter
    def learning_rate_denominator(self, learning_rate_denominator):
        """Sets the learning_rate_denominator of this TrainingConfig.

        See `learningRateNumerator`.

        :param learning_rate_denominator: The learning_rate_denominator of this TrainingConfig.
        :type: int
        """

        self._learning_rate_denominator = learning_rate_denominator

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(TrainingConfig, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TrainingConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
